@@ -1,6 +1,9 @@
 /* { dg-do compile } */
 /* { dg-options "-O3 -fdump-tree-fnsplit" } */
-int test2(a)
+
+void do_something_big (void);
+
+int test2(int a)
 {
    if (a<100)
      return 1;
@@ -20,6 +23,7 @@ int test2(a)
    return 0;
 }
 
+void
 test()
 {
   test2(10);
