@@ -3,9 +3,11 @@
 
 /* Test inserting in a block that does not contain a division.  */
 
-extern int f2();
+extern int g();
 
-double f1(double y, double z, double w)
+double m, n, o;
+
+void f1(double y, double z, double w)
 {
   double b, c, d, e, f;
 
@@ -18,7 +20,9 @@ double f1(double y, double z, double w)
   e = c / y;
   f = 1 / y;
 
-  return d + e + f;
+  m = d;
+  n = e;
+  o = f;
 }
 
 /* { dg-final { scan-tree-dump-times " / " 1 "recip" } } */
