@@ -1,6 +1,4 @@
-#import "../../objc-obj-c++-shared/Object1.h"
 #include <objc/objc.h>
-#include <objc/objc-api.h>
 
 @interface MyObject
 {
