@@ -1,7 +1,7 @@
 /* Contributed by David Ayers - Fri Jun  4 03:16:17 BST 2004 */
 /* Test that a protocol is not equal to something which is not a protocol.  */
-
-#include "../../objc-obj-c++-shared/Protocol1.h"
+#include "../../objc-obj-c++-shared/runtime.h"
+#include <objc/Protocol.h>
 
 @protocol Foo
 - (void)foo;
@@ -10,11 +10,7 @@
 int main (void)
 {
   /* A Protocol object should not be equal to a Class object.  */
-#ifdef NEXT_OBJC_USE_NEW_INTERFACE
-  if (protocol_isEqual (@protocol(Foo), objc_getClass("Protocol")))
-#else
-  if ([@protocol(Foo) isEqual: [Protocol class]])
-#endif
+  if (protocol_isEqual (@protocol(Foo), (id)objc_getClass("Protocol")))
     {
       abort ();
     }
