@@ -1,5 +1,5 @@
 /* GCC instrumentation plugin for ThreadSanitizer.
-   Copyright (C) 2011-2013 Free Software Foundation, Inc.
+   Copyright (C) 2011-2015 Free Software Foundation, Inc.
    Contributed by Dmitry Vyukov <dvyukov@google.com>
 
 This file is part of GCC.
@@ -22,24 +22,68 @@ along with GCC; see the file COPYING3.  If not see
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
+#include "hash-set.h"
+#include "machmode.h"
+#include "vec.h"
+#include "double-int.h"
+#include "input.h"
+#include "alias.h"
+#include "symtab.h"
+#include "options.h"
+#include "wide-int.h"
+#include "inchash.h"
 #include "tree.h"
-#include "intl.h"
+#include "fold-const.h"
+#include "hashtab.h"
 #include "tm.h"
-#include "basic-block.h"
-#include "gimple.h"
+#include "hard-reg-set.h"
 #include "function.h"
-#include "tree-flow.h"
+#include "rtl.h"
+#include "flags.h"
+#include "statistics.h"
+#include "real.h"
+#include "fixed-value.h"
+#include "insn-config.h"
+#include "expmed.h"
+#include "dojump.h"
+#include "explow.h"
+#include "calls.h"
+#include "emit-rtl.h"
+#include "varasm.h"
+#include "stmt.h"
+#include "expr.h"
+#include "intl.h"
+#include "predict.h"
+#include "dominance.h"
+#include "cfg.h"
+#include "basic-block.h"
+#include "tree-ssa-alias.h"
+#include "internal-fn.h"
+#include "gimple-expr.h"
+#include "is-a.h"
+#include "gimple.h"
+#include "gimplify.h"
+#include "gimple-iterator.h"
+#include "gimplify-me.h"
+#include "gimple-ssa.h"
+#include "hash-map.h"
+#include "plugin-api.h"
+#include "ipa-ref.h"
+#include "cgraph.h"
+#include "tree-cfg.h"
+#include "stringpool.h"
+#include "tree-ssanames.h"
 #include "tree-pass.h"
 #include "tree-iterator.h"
 #include "langhooks.h"
 #include "output.h"
-#include "options.h"
 #include "target.h"
-#include "cgraph.h"
 #include "diagnostic.h"
 #include "tree-ssa-propagate.h"
+#include "tree-ssa-loop-ivopts.h"
 #include "tsan.h"
 #include "asan.h"
+#include "builtins.h"
 
 /* Number of instrumented memory accesses in the current function.  */
 
@@ -99,63 +143,113 @@ instrument_expr (gimple_stmt_iterator gsi, tree expr, bool is_write)
   gimple stmt, g;
   gimple_seq seq;
   location_t loc;
+  unsigned int align;
 
   size = int_size_in_bytes (TREE_TYPE (expr));
-  if (size == -1)
+  if (size <= 0)
     return false;
 
-  /* For now just avoid instrumenting bit field acceses.
-     TODO: handle bit-fields as if touching the whole field.  */
   HOST_WIDE_INT bitsize, bitpos;
   tree offset;
-  enum machine_mode mode;
+  machine_mode mode;
   int volatilep = 0, unsignedp = 0;
   base = get_inner_reference (expr, &bitsize, &bitpos, &offset,
 			      &mode, &unsignedp, &volatilep, false);
 
   /* No need to instrument accesses to decls that don't escape,
      they can't escape to other threads then.  */
-  if (DECL_P (base))
+  if (DECL_P (base) && !is_global_var (base))
     {
       struct pt_solution pt;
       memset (&pt, 0, sizeof (pt));
       pt.escaped = 1;
       pt.ipa_escaped = flag_ipa_pta != 0;
-      pt.nonlocal = 1;
       if (!pt_solution_includes (&pt, base))
 	return false;
-      if (!is_global_var (base) && !may_be_aliased (base))
+      if (!may_be_aliased (base))
 	return false;
     }
 
-  if (TREE_READONLY (base))
-    return false;
-
-  if (bitpos % (size * BITS_PER_UNIT)
-      || bitsize != size * BITS_PER_UNIT)
+  if (TREE_READONLY (base)
+      || (TREE_CODE (base) == VAR_DECL
+	  && DECL_HARD_REGISTER (base)))
     return false;
 
   stmt = gsi_stmt (gsi);
   loc = gimple_location (stmt);
   rhs = is_vptr_store (stmt, expr, is_write);
-  gcc_checking_assert (rhs != NULL || is_gimple_addressable (expr));
-  expr_ptr = build_fold_addr_expr (unshare_expr (expr));
-  seq = NULL;
-  if (!is_gimple_val (expr_ptr))
+
+  if ((TREE_CODE (expr) == COMPONENT_REF
+       && DECL_BIT_FIELD_TYPE (TREE_OPERAND (expr, 1)))
+      || TREE_CODE (expr) == BIT_FIELD_REF)
     {
-      g = gimple_build_assign (make_ssa_name (TREE_TYPE (expr_ptr), NULL),
-			       expr_ptr);
-      expr_ptr = gimple_assign_lhs (g);
-      gimple_set_location (g, loc);
-      gimple_seq_add_stmt_without_update (&seq, g);
+      base = TREE_OPERAND (expr, 0);
+      if (TREE_CODE (expr) == COMPONENT_REF)
+	{
+	  expr = TREE_OPERAND (expr, 1);
+	  if (is_write && DECL_BIT_FIELD_REPRESENTATIVE (expr))
+	    expr = DECL_BIT_FIELD_REPRESENTATIVE (expr);
+	  if (!tree_fits_uhwi_p (DECL_FIELD_OFFSET (expr))
+	      || !tree_fits_uhwi_p (DECL_FIELD_BIT_OFFSET (expr))
+	      || !tree_fits_uhwi_p (DECL_SIZE (expr)))
+	    return false;
+	  bitpos = tree_to_uhwi (DECL_FIELD_OFFSET (expr)) * BITS_PER_UNIT
+		   + tree_to_uhwi (DECL_FIELD_BIT_OFFSET (expr));
+	  bitsize = tree_to_uhwi (DECL_SIZE (expr));
+	}
+      else
+	{
+	  if (!tree_fits_uhwi_p (TREE_OPERAND (expr, 2))
+	      || !tree_fits_uhwi_p (TREE_OPERAND (expr, 1)))
+	    return false;
+	  bitpos = tree_to_uhwi (TREE_OPERAND (expr, 2));
+	  bitsize = tree_to_uhwi (TREE_OPERAND (expr, 1));
+	}
+      if (bitpos < 0 || bitsize <= 0)
+	return false;
+      size = (bitpos % BITS_PER_UNIT + bitsize + BITS_PER_UNIT - 1)
+	     / BITS_PER_UNIT;
+      if (may_be_nonaddressable_p (base))
+	return false;
+      align = get_object_alignment (base);
+      if (align < BITS_PER_UNIT)
+	return false;
+      bitpos = bitpos & ~(BITS_PER_UNIT - 1);
+      if ((align - 1) & bitpos)
+	{
+	  align = (align - 1) & bitpos;
+	  align = align & -align;
+	}
+      expr = build_fold_addr_expr (unshare_expr (base));
+      expr = build2 (MEM_REF, char_type_node, expr,
+		     build_int_cst (TREE_TYPE (expr), bitpos / BITS_PER_UNIT));
+      expr_ptr = build_fold_addr_expr (expr);
     }
-  if (rhs == NULL)
+  else
+    {
+      if (may_be_nonaddressable_p (expr))
+	return false;
+      align = get_object_alignment (expr);
+      if (align < BITS_PER_UNIT)
+	return false;
+      expr_ptr = build_fold_addr_expr (unshare_expr (expr));
+    }
+  expr_ptr = force_gimple_operand (expr_ptr, &seq, true, NULL_TREE);
+  if ((size & (size - 1)) != 0 || size > 16
+      || align < MIN (size, 8) * BITS_PER_UNIT)
+    {
+      builtin_decl = builtin_decl_implicit (is_write
+					    ? BUILT_IN_TSAN_WRITE_RANGE
+					    : BUILT_IN_TSAN_READ_RANGE);
+      g = gimple_build_call (builtin_decl, 2, expr_ptr, size_int (size));
+    }
+  else if (rhs == NULL)
     g = gimple_build_call (get_memory_access_decl (is_write, size),
 			   1, expr_ptr);
   else
     {
       builtin_decl = builtin_decl_implicit (BUILT_IN_TSAN_VPTR_UPDATE);
-      g = gimple_build_call (builtin_decl, 1, expr_ptr);
+      g = gimple_build_call (builtin_decl, 2, expr_ptr, unshare_expr (rhs));
     }
   gimple_set_location (g, loc);
   gimple_seq_add_stmt_without_update (&seq, g);
@@ -195,7 +289,7 @@ enum tsan_atomic_action
 
 /* Table how to map sync/atomic builtins to their corresponding
    tsan equivalents.  */
-static struct tsan_map_atomic
+static const struct tsan_map_atomic
 {
   enum built_in_function fcode, tsan_fcode;
   enum tsan_atomic_action action;
@@ -440,9 +534,8 @@ instrument_builtin_call (gimple_stmt_iterator *gsi)
 	  case check_last:
 	  case fetch_op:
 	    last_arg = gimple_call_arg (stmt, num - 1);
-	    if (!host_integerp (last_arg, 1)
-		|| (unsigned HOST_WIDE_INT) tree_low_cst (last_arg, 1)
-		   > MEMMODEL_SEQ_CST)
+	    if (!tree_fits_uhwi_p (last_arg)
+		|| memmodel_base (tree_to_uhwi (last_arg)) >= MEMMODEL_LAST)
 	      return;
 	    gimple_call_set_fndecl (stmt, decl);
 	    update_stmt (stmt);
@@ -476,30 +569,24 @@ instrument_builtin_call (gimple_stmt_iterator *gsi)
 		if (!useless_type_conversion_p (TREE_TYPE (lhs),
 						TREE_TYPE (args[1])))
 		  {
-		    tree var = make_ssa_name (TREE_TYPE (lhs), NULL);
-		    g = gimple_build_assign_with_ops (NOP_EXPR, var,
-						      args[1], NULL_TREE);
+		    tree var = make_ssa_name (TREE_TYPE (lhs));
+		    g = gimple_build_assign (var, NOP_EXPR, args[1]);
 		    gsi_insert_after (gsi, g, GSI_NEW_STMT);
 		    args[1] = var;
 		  }
-		gimple_call_set_lhs (stmt,
-				     make_ssa_name (TREE_TYPE (lhs), NULL));
+		gimple_call_set_lhs (stmt, make_ssa_name (TREE_TYPE (lhs)));
 		/* BIT_NOT_EXPR stands for NAND.  */
 		if (tsan_atomic_table[i].code == BIT_NOT_EXPR)
 		  {
-		    tree var = make_ssa_name (TREE_TYPE (lhs), NULL);
-		    g = gimple_build_assign_with_ops (BIT_AND_EXPR, var,
-						      gimple_call_lhs (stmt),
-						      args[1]);
+		    tree var = make_ssa_name (TREE_TYPE (lhs));
+		    g = gimple_build_assign (var, BIT_AND_EXPR,
+					     gimple_call_lhs (stmt), args[1]);
 		    gsi_insert_after (gsi, g, GSI_NEW_STMT);
-		    g = gimple_build_assign_with_ops (BIT_NOT_EXPR, lhs, var,
-						      NULL_TREE);
+		    g = gimple_build_assign (lhs, BIT_NOT_EXPR, var);
 		  }
 		else
-		  g = gimple_build_assign_with_ops (tsan_atomic_table[i].code,
-						    lhs,
-						    gimple_call_lhs (stmt),
-						    args[1]);
+		  g = gimple_build_assign (lhs, tsan_atomic_table[i].code,
+					   gimple_call_lhs (stmt), args[1]);
 		update_stmt (stmt);
 		gsi_insert_after (gsi, g, GSI_NEW_STMT);
 	      }
@@ -512,13 +599,11 @@ instrument_builtin_call (gimple_stmt_iterator *gsi)
 	    gcc_assert (num == 6);
 	    for (j = 0; j < 6; j++)
 	      args[j] = gimple_call_arg (stmt, j);
-	    if (!host_integerp (args[4], 1)
-		|| (unsigned HOST_WIDE_INT) tree_low_cst (args[4], 1)
-		   > MEMMODEL_SEQ_CST)
+	    if (!tree_fits_uhwi_p (args[4])
+		|| memmodel_base (tree_to_uhwi (args[4])) >= MEMMODEL_LAST)
 	      return;
-	    if (!host_integerp (args[5], 1)
-		|| (unsigned HOST_WIDE_INT) tree_low_cst (args[5], 1)
-		   > MEMMODEL_SEQ_CST)
+	    if (!tree_fits_uhwi_p (args[5])
+		|| memmodel_base (tree_to_uhwi (args[5])) >= MEMMODEL_LAST)
 	      return;
 	    update_gimple_call (gsi, decl, 5, args[0], args[1], args[2],
 				args[4], args[5]);
@@ -530,15 +615,13 @@ instrument_builtin_call (gimple_stmt_iterator *gsi)
 	      args[j] = gimple_call_arg (stmt, j);
 	    t = TYPE_ARG_TYPES (TREE_TYPE (decl));
 	    t = TREE_VALUE (TREE_CHAIN (TREE_CHAIN (t)));
-	    t = create_tmp_var (t, NULL);
+	    t = create_tmp_var (t);
 	    mark_addressable (t);
 	    if (!useless_type_conversion_p (TREE_TYPE (t),
 					    TREE_TYPE (args[1])))
 	      {
-		g = gimple_build_assign_with_ops (NOP_EXPR,
-						  make_ssa_name (TREE_TYPE (t),
-								 NULL),
-						  args[1], NULL_TREE);
+		g = gimple_build_assign (make_ssa_name (TREE_TYPE (t)),
+					 NOP_EXPR, args[1]);
 		gsi_insert_before (gsi, g, GSI_SAME_STMT);
 		args[1] = gimple_assign_lhs (g);
 	      }
@@ -555,15 +638,13 @@ instrument_builtin_call (gimple_stmt_iterator *gsi)
 	      {
 		tree cond;
 		stmt = gsi_stmt (*gsi);
-		g = gimple_build_assign (make_ssa_name (TREE_TYPE (t), NULL),
-					 t);
+		g = gimple_build_assign (make_ssa_name (TREE_TYPE (t)), t);
 		gsi_insert_after (gsi, g, GSI_NEW_STMT);
 		t = make_ssa_name (TREE_TYPE (TREE_TYPE (decl)), stmt);
 		cond = build2 (NE_EXPR, boolean_type_node, t,
 			       build_int_cst (TREE_TYPE (t), 0));
-		g = gimple_build_assign_with_ops (COND_EXPR, lhs, cond,
-						  args[1],
-						  gimple_assign_lhs (g));
+		g = gimple_build_assign (lhs, COND_EXPR, cond, args[1],
+					 gimple_assign_lhs (g));
 		gimple_call_set_lhs (stmt, t);
 		update_stmt (stmt);
 		gsi_insert_after (gsi, g, GSI_NEW_STMT);
@@ -599,7 +680,11 @@ instrument_gimple (gimple_stmt_iterator *gsi)
       && (gimple_call_fndecl (stmt)
 	  != builtin_decl_implicit (BUILT_IN_TSAN_INIT)))
     {
-      if (is_gimple_builtin_call (stmt))
+      /* All functions with function call will have exit instrumented,
+	 therefore no function calls other than __tsan_func_exit
+	 shall appear in the functions.  */
+      gimple_call_set_tail (as_a <gcall *> (stmt), false);
+      if (gimple_call_builtin_p (stmt, BUILT_IN_NORMAL))
 	instrument_builtin_call (gsi);
       return true;
     }
@@ -620,49 +705,19 @@ instrument_gimple (gimple_stmt_iterator *gsi)
   return instrumented;
 }
 
-/* Instruments all interesting memory accesses in the current function.
-   Return true if func entry/exit should be instrumented.  */
-
-static bool
-instrument_memory_accesses (void)
-{
-  basic_block bb;
-  gimple_stmt_iterator gsi;
-  bool fentry_exit_instrument = false;
-
-  FOR_EACH_BB (bb)
-    for (gsi = gsi_start_bb (bb); !gsi_end_p (gsi); gsi_next (&gsi))
-      fentry_exit_instrument |= instrument_gimple (&gsi);
-  return fentry_exit_instrument;
-}
-
-/* Instruments function entry.  */
+/* Replace TSAN_FUNC_EXIT internal call with function exit tsan builtin.  */
 
 static void
-instrument_func_entry (void)
+replace_func_exit (gimple stmt)
 {
-  basic_block succ_bb;
-  gimple_stmt_iterator gsi;
-  tree ret_addr, builtin_decl;
-  gimple g;
-
-  succ_bb = single_succ (ENTRY_BLOCK_PTR);
-  gsi = gsi_after_labels (succ_bb);
-
-  builtin_decl = builtin_decl_implicit (BUILT_IN_RETURN_ADDRESS);
-  g = gimple_build_call (builtin_decl, 1, integer_zero_node);
-  ret_addr = make_ssa_name (ptr_type_node, NULL);
-  gimple_call_set_lhs (g, ret_addr);
-  gimple_set_location (g, cfun->function_start_locus);
-  gsi_insert_before (&gsi, g, GSI_SAME_STMT);
-
-  builtin_decl =  builtin_decl_implicit (BUILT_IN_TSAN_FUNC_ENTRY);
-  g = gimple_build_call (builtin_decl, 1, ret_addr);
-  gimple_set_location (g, cfun->function_start_locus);
-  gsi_insert_before (&gsi, g, GSI_SAME_STMT);
+  tree builtin_decl = builtin_decl_implicit (BUILT_IN_TSAN_FUNC_EXIT);
+  gimple g = gimple_build_call (builtin_decl, 0);
+  gimple_set_location (g, cfun->function_end_locus);
+  gimple_stmt_iterator gsi = gsi_for_stmt (stmt);
+  gsi_replace (&gsi, g, true);
 }
 
-/* Instruments function exits.  */
+/* Instrument function exit.  Used when TSAN_FUNC_EXIT does not exist.  */
 
 static void
 instrument_func_exit (void)
@@ -676,12 +731,13 @@ instrument_func_exit (void)
   edge_iterator ei;
 
   /* Find all function exits.  */
-  exit_bb = EXIT_BLOCK_PTR;
+  exit_bb = EXIT_BLOCK_PTR_FOR_FN (cfun);
   FOR_EACH_EDGE (e, ei, exit_bb->preds)
     {
       gsi = gsi_last_bb (e->src);
       stmt = gsi_stmt (gsi);
-      gcc_assert (gimple_code (stmt) == GIMPLE_RETURN);
+      gcc_assert (gimple_code (stmt) == GIMPLE_RETURN
+		  || gimple_call_builtin_p (stmt, BUILT_IN_RETURN));
       loc = gimple_location (stmt);
       builtin_decl = builtin_decl_implicit (BUILT_IN_TSAN_FUNC_EXIT);
       g = gimple_build_call (builtin_decl, 0);
@@ -690,6 +746,75 @@ instrument_func_exit (void)
     }
 }
 
+/* Instruments all interesting memory accesses in the current function.
+   Return true if func entry/exit should be instrumented.  */
+
+static bool
+instrument_memory_accesses (void)
+{
+  basic_block bb;
+  gimple_stmt_iterator gsi;
+  bool fentry_exit_instrument = false;
+  bool func_exit_seen = false;
+  auto_vec<gimple> tsan_func_exits;
+
+  FOR_EACH_BB_FN (bb, cfun)
+    for (gsi = gsi_start_bb (bb); !gsi_end_p (gsi); gsi_next (&gsi))
+      {
+	gimple stmt = gsi_stmt (gsi);
+	if (is_gimple_call (stmt)
+	    && gimple_call_internal_p (stmt)
+	    && gimple_call_internal_fn (stmt) == IFN_TSAN_FUNC_EXIT)
+	  {
+	    if (fentry_exit_instrument)
+	      replace_func_exit (stmt);
+	    else
+	      tsan_func_exits.safe_push (stmt);
+	    func_exit_seen = true;
+	  }
+	else
+	  fentry_exit_instrument |= instrument_gimple (&gsi);
+      }
+  unsigned int i;
+  gimple stmt;
+  FOR_EACH_VEC_ELT (tsan_func_exits, i, stmt)
+    if (fentry_exit_instrument)
+      replace_func_exit (stmt);
+    else
+      {
+	gsi = gsi_for_stmt (stmt);
+	gsi_remove (&gsi, true);
+      }
+  if (fentry_exit_instrument && !func_exit_seen)
+    instrument_func_exit ();
+  return fentry_exit_instrument;
+}
+
+/* Instruments function entry.  */
+
+static void
+instrument_func_entry (void)
+{
+  tree ret_addr, builtin_decl;
+  gimple g;
+  gimple_seq seq = NULL;
+
+  builtin_decl = builtin_decl_implicit (BUILT_IN_RETURN_ADDRESS);
+  g = gimple_build_call (builtin_decl, 1, integer_zero_node);
+  ret_addr = make_ssa_name (ptr_type_node);
+  gimple_call_set_lhs (g, ret_addr);
+  gimple_set_location (g, cfun->function_start_locus);
+  gimple_seq_add_stmt_without_update (&seq, g);
+
+  builtin_decl = builtin_decl_implicit (BUILT_IN_TSAN_FUNC_ENTRY);
+  g = gimple_build_call (builtin_decl, 1, ret_addr);
+  gimple_set_location (g, cfun->function_start_locus);
+  gimple_seq_add_stmt_without_update (&seq, g);
+
+  edge e = single_succ_edge (ENTRY_BLOCK_PTR_FOR_FN (cfun));
+  gsi_insert_seq_on_edge_immediate (e, seq);
+}
+
 /* ThreadSanitizer instrumentation pass.  */
 
 static unsigned
@@ -697,21 +822,10 @@ tsan_pass (void)
 {
   initialize_sanitizer_builtins ();
   if (instrument_memory_accesses ())
-    {
-      instrument_func_entry ();
-      instrument_func_exit ();
-    }
+    instrument_func_entry ();
   return 0;
 }
 
-/* The pass's gate.  */
-
-static bool
-tsan_gate (void)
-{
-  return flag_tsan != 0;
-}
-
 /* Inserts __tsan_init () into the list of CTORs.  */
 
 void
@@ -729,48 +843,87 @@ tsan_finish_file (void)
 
 /* The pass descriptor.  */
 
-struct gimple_opt_pass pass_tsan =
+namespace {
+
+const pass_data pass_data_tsan =
 {
- {
-  GIMPLE_PASS,
-  "tsan",				/* name  */
-  OPTGROUP_NONE,			/* optinfo_flags */
-  tsan_gate,				/* gate  */
-  tsan_pass,				/* execute  */
-  NULL,					/* sub  */
-  NULL,					/* next  */
-  0,					/* static_pass_number  */
-  TV_NONE,				/* tv_id  */
-  PROP_ssa | PROP_cfg,			/* properties_required  */
-  0,					/* properties_provided  */
-  0,					/* properties_destroyed  */
-  0,					/* todo_flags_start  */
-  TODO_verify_all | TODO_update_ssa	/* todo_flags_finish  */
- }
+  GIMPLE_PASS, /* type */
+  "tsan", /* name */
+  OPTGROUP_NONE, /* optinfo_flags */
+  TV_NONE, /* tv_id */
+  ( PROP_ssa | PROP_cfg ), /* properties_required */
+  0, /* properties_provided */
+  0, /* properties_destroyed */
+  0, /* todo_flags_start */
+  TODO_update_ssa, /* todo_flags_finish */
 };
 
-static bool
-tsan_gate_O0 (void)
+class pass_tsan : public gimple_opt_pass
 {
-  return flag_tsan != 0 && !optimize;
+public:
+  pass_tsan (gcc::context *ctxt)
+    : gimple_opt_pass (pass_data_tsan, ctxt)
+  {}
+
+  /* opt_pass methods: */
+  opt_pass * clone () { return new pass_tsan (m_ctxt); }
+  virtual bool gate (function *)
+{
+  return ((flag_sanitize & SANITIZE_THREAD) != 0
+	  && !lookup_attribute ("no_sanitize_thread",
+                                DECL_ATTRIBUTES (current_function_decl)));
 }
 
-struct gimple_opt_pass pass_tsan_O0 =
+  virtual unsigned int execute (function *) { return tsan_pass (); }
+
+}; // class pass_tsan
+
+} // anon namespace
+
+gimple_opt_pass *
+make_pass_tsan (gcc::context *ctxt)
 {
- {
-  GIMPLE_PASS,
-  "tsan0",				/* name  */
-  OPTGROUP_NONE,			/* optinfo_flags */
-  tsan_gate_O0,				/* gate  */
-  tsan_pass,				/* execute  */
-  NULL,					/* sub  */
-  NULL,					/* next  */
-  0,					/* static_pass_number  */
-  TV_NONE,				/* tv_id  */
-  PROP_ssa | PROP_cfg,			/* properties_required  */
-  0,					/* properties_provided  */
-  0,					/* properties_destroyed  */
-  0,					/* todo_flags_start  */
-  TODO_verify_all | TODO_update_ssa	/* todo_flags_finish  */
- }
+  return new pass_tsan (ctxt);
+}
+
+namespace {
+
+const pass_data pass_data_tsan_O0 =
+{
+  GIMPLE_PASS, /* type */
+  "tsan0", /* name */
+  OPTGROUP_NONE, /* optinfo_flags */
+  TV_NONE, /* tv_id */
+  ( PROP_ssa | PROP_cfg ), /* properties_required */
+  0, /* properties_provided */
+  0, /* properties_destroyed */
+  0, /* todo_flags_start */
+  TODO_update_ssa, /* todo_flags_finish */
 };
+
+class pass_tsan_O0 : public gimple_opt_pass
+{
+public:
+  pass_tsan_O0 (gcc::context *ctxt)
+    : gimple_opt_pass (pass_data_tsan_O0, ctxt)
+  {}
+
+  /* opt_pass methods: */
+  virtual bool gate (function *)
+    {
+      return ((flag_sanitize & SANITIZE_THREAD) != 0 && !optimize
+	      && !lookup_attribute ("no_sanitize_thread",
+				    DECL_ATTRIBUTES (current_function_decl)));
+    }
+
+  virtual unsigned int execute (function *) { return tsan_pass (); }
+
+}; // class pass_tsan_O0
+
+} // anon namespace
+
+gimple_opt_pass *
+make_pass_tsan_O0 (gcc::context *ctxt)
+{
+  return new pass_tsan_O0 (ctxt);
+}
