@@ -89,7 +89,7 @@ var fileListenerTests = []struct {
 
 func TestFileListener(t *testing.T) {
 	switch runtime.GOOS {
-	case "windows":
+	case "nacl", "windows":
 		t.Skipf("skipping test on %q", runtime.GOOS)
 	}
 
@@ -174,12 +174,14 @@ var filePacketConnTests = []struct {
 
 	{net: "udp6", addr: "[::1]", ipv6: true},
 
+	{net: "ip4:icmp", addr: "127.0.0.1"},
+
 	{net: "unixgram", addr: "@gotest3/net", linux: true},
 }
 
 func TestFilePacketConn(t *testing.T) {
 	switch runtime.GOOS {
-	case "plan9", "windows":
+	case "nacl", "plan9", "windows":
 		t.Skipf("skipping test on %q", runtime.GOOS)
 	}
 
@@ -187,6 +189,10 @@ func TestFilePacketConn(t *testing.T) {
 		if skipServerTest(tt.net, "unixgram", tt.addr, tt.ipv6, false, tt.linux) {
 			continue
 		}
+		if os.Getuid() != 0 && tt.net == "ip4:icmp" {
+			t.Log("skipping test; must be root")
+			continue
+		}
 		testFilePacketConnListen(t, tt.net, tt.addr)
 		switch tt.addr {
 		case "", "0.0.0.0", "[::ffff:0.0.0.0]", "[::]":
