@@ -19,12 +19,6 @@
 
 namespace __tsan {
 
-ScopedInRtl::ScopedInRtl() {
-}
-
-ScopedInRtl::~ScopedInRtl() {
-}
-
 uptr GetShadowMemoryConsumption() {
   return 0;
 }
@@ -32,12 +26,14 @@ uptr GetShadowMemoryConsumption() {
 void FlushShadowMemory() {
 }
 
-const char *InitializePlatform() {
-  return GetEnv(kTsanOptionsEnv);
+void WriteMemoryProfile(char *buf, uptr buf_size, uptr nthread, uptr nlive) {
 }
 
-void FinalizePlatform() {
-  fflush(0);
+uptr GetRSS() {
+  return 0;
+}
+
+void InitializePlatform() {
 }
 
 }  // namespace __tsan
