@@ -1,11 +1,11 @@
 // std::time_get, std::time_put implementation, generic version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+// Copyright (C) 2001-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -13,19 +13,19 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 
-// You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// Under Section 7 of GPL version 3, you are granted additional
+// permissions described in the GCC Runtime Library Exception, version
+// 3.1, as published by the Free Software Foundation.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// You should have received a copy of the GNU General Public License and
+// a copy of the GCC Runtime Library Exception along with this program;
+// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
+// <http://www.gnu.org/licenses/>.
+
+/** @file bits/time_members.h
+ *  This is an internal header file, included by other library headers.
+ *  Do not attempt to use it directly. @headername{locale}
+ */
 
 //
 // ISO C++ 14882: 22.2.5.1.2 - time_get functions
@@ -34,9 +34,13 @@
 
 // Written by Benjamin Kosnik <bkoz@redhat.com>
 
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_VERSION
+
   template<typename _CharT>
     __timepunct<_CharT>::__timepunct(size_t __refs) 
-    : facet(__refs), _M_data(NULL)
+    : facet(__refs), _M_data(0)
     { 
       _M_name_timepunct = _S_get_c_name();
       _M_initialize_timepunct(); 
@@ -53,18 +57,24 @@
   template<typename _CharT>
     __timepunct<_CharT>::__timepunct(__c_locale __cloc, const char* __s, 
 				     size_t __refs) 
-    : facet(__refs), _M_data(NULL)
-    { 
-      const size_t __len = std::strlen(__s) + 1;
-      char* __tmp = new char[__len];
-      std::memcpy(__tmp, __s, __len);
-      _M_name_timepunct = __tmp;
+    : facet(__refs), _M_data(0)
+    {
+      if (__builtin_strcmp(__s, _S_get_c_name()) != 0)
+	{
+	  const size_t __len = __builtin_strlen(__s) + 1;
+	  char* __tmp = new char[__len];
+	  __builtin_memcpy(__tmp, __s, __len);
+	  _M_name_timepunct = __tmp;
+	}
+      else
+	_M_name_timepunct = _S_get_c_name();
 
-      try
+      __try
 	{ _M_initialize_timepunct(__cloc); }
-      catch(...)
-	{ 
-	  delete [] _M_name_timepunct;
+      __catch(...)
+	{
+	  if (_M_name_timepunct != _S_get_c_name())
+	    delete [] _M_name_timepunct;
 	  __throw_exception_again;
 	}
     }
@@ -77,3 +87,6 @@
       delete _M_data;
       _S_destroy_c_locale(_M_c_locale_timepunct); 
     }
+
+_GLIBCXX_END_NAMESPACE_VERSION
+} // namespace
