@@ -1,6 +1,6 @@
 // -*- C++ -*-
 
-// Copyright (C) 2005, 2006, 2009, 2010 Free Software Foundation, Inc.
+// Copyright (C) 2005-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the terms
@@ -34,7 +34,7 @@
 // warranty.
 
 /**
- * @file erase_fn_imps.hpp
+ * @file thin_heap_/erase_fn_imps.hpp
  * Contains an implementation for thin_heap_.
  */
 
@@ -43,19 +43,15 @@ void
 PB_DS_CLASS_C_DEC::
 pop()
 {
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-    _GLIBCXX_DEBUG_ASSERT(!base_type::empty());
-
+  PB_DS_ASSERT_VALID((*this))
+  _GLIBCXX_DEBUG_ASSERT(!base_type::empty());
   _GLIBCXX_DEBUG_ASSERT(m_p_max != 0);
 
   node_pointer p_nd = m_p_max;
-
   remove_max_node();
-
   base_type::actual_erase_node(p_nd);
-
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-    }
+  PB_DS_ASSERT_VALID((*this))
+}
 
 PB_DS_CLASS_T_DEC
 inline void
@@ -63,7 +59,6 @@ PB_DS_CLASS_C_DEC::
 remove_max_node()
 {
   to_aux_except_max();
-
   make_from_aux();
 }
 
@@ -73,39 +68,29 @@ PB_DS_CLASS_C_DEC::
 to_aux_except_max()
 {
   node_pointer p_add = base_type::m_p_root;
-
   while (p_add != m_p_max)
     {
       node_pointer p_next_add = p_add->m_p_next_sibling;
-
       add_to_aux(p_add);
-
       p_add = p_next_add;
     }
 
   p_add = m_p_max->m_p_l_child;
-
   while (p_add != 0)
     {
       node_pointer p_next_add = p_add->m_p_next_sibling;
-
-      p_add->m_metadata = p_add->m_p_l_child == 0?
-	0 :
-	p_add->m_p_l_child->m_metadata + 1;
+      p_add->m_metadata = p_add->m_p_l_child == 0 ?
+	0 : p_add->m_p_l_child->m_metadata + 1;
 
       add_to_aux(p_add);
-
       p_add = p_next_add;
     }
 
   p_add = m_p_max->m_p_next_sibling;
-
   while (p_add != 0)
     {
       node_pointer p_next_add = p_add->m_p_next_sibling;
-
       add_to_aux(p_add);
-
       p_add = p_next_add;
     }
 }
@@ -116,22 +101,18 @@ PB_DS_CLASS_C_DEC::
 add_to_aux(node_pointer p_nd)
 {
   size_type r = p_nd->m_metadata;
-
   while (m_a_aux[r] != 0)
     {
       _GLIBCXX_DEBUG_ASSERT(p_nd->m_metadata < rank_bound());
-
       if (Cmp_Fn::operator()(m_a_aux[r]->m_value, p_nd->m_value))
 	make_child_of(m_a_aux[r], p_nd);
       else
-        {
+	{
 	  make_child_of(p_nd, m_a_aux[r]);
-
 	  p_nd = m_a_aux[r];
-        }
+	}
 
       m_a_aux[r] = 0;
-
       ++r;
     }
 
@@ -150,7 +131,6 @@ make_child_of(node_pointer p_nd, node_pointer p_new_parent)
 		   m_a_aux[p_nd->m_metadata] == p_new_parent);
 
   ++p_new_parent->m_metadata;
-
   base_type::make_child_of(p_nd, p_new_parent);
 }
 
@@ -160,25 +140,20 @@ PB_DS_CLASS_C_DEC::
 make_from_aux()
 {
   base_type::m_p_root = m_p_max = 0;
-
   const size_type rnk_bnd = rank_bound();
-
   size_type i = 0;
-
   while (i < rnk_bnd)
     {
       if (m_a_aux[i] != 0)
-        {
+	{
 	  make_root_and_link(m_a_aux[i]);
-
 	  m_a_aux[i] = 0;
-        }
-
+	}
       ++i;
     }
 
-  _GLIBCXX_DEBUG_ONLY(assert_aux_null();)
-    }
+  PB_DS_ASSERT_AUX_NULL((*this))
+}
 
 PB_DS_CLASS_T_DEC
 inline void
@@ -190,7 +165,6 @@ remove_node(node_pointer p_nd)
     p_parent = base_type::parent(p_parent);
 
   base_type::bubble_to_top(p_nd);
-
   m_p_max = p_nd;
 
   node_pointer p_fix = base_type::m_p_root;
@@ -209,7 +183,6 @@ PB_DS_CLASS_C_DEC::
 clear()
 {
   base_type::clear();
-
   m_p_max = 0;
 }
 
@@ -218,17 +191,14 @@ void
 PB_DS_CLASS_C_DEC::
 erase(point_iterator it)
 {
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-    _GLIBCXX_DEBUG_ASSERT(!base_type::empty());
+  PB_DS_ASSERT_VALID((*this))
+  _GLIBCXX_DEBUG_ASSERT(!base_type::empty());
 
   node_pointer p_nd = it.m_p_nd;
-
   remove_node(p_nd);
-
   base_type::actual_erase_node(p_nd);
-
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-    }
+  PB_DS_ASSERT_VALID((*this))
+}
 
 PB_DS_CLASS_T_DEC
 template<typename Pred>
@@ -236,48 +206,35 @@ typename PB_DS_CLASS_C_DEC::size_type
 PB_DS_CLASS_C_DEC::
 erase_if(Pred pred)
 {
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-
-    if (base_type::empty())
-      {
-        _GLIBCXX_DEBUG_ONLY(assert_valid();)
-
-	  return 0;
-      }
+  PB_DS_ASSERT_VALID((*this))
+  if (base_type::empty())
+    {
+      PB_DS_ASSERT_VALID((*this))
+      return 0;
+    }
 
   base_type::to_linked_list();
-
   node_pointer p_out = base_type::prune(pred);
-
   size_type ersd = 0;
-
   while (p_out != 0)
     {
       ++ersd;
-
       node_pointer p_next = p_out->m_p_next_sibling;
-
       base_type::actual_erase_node(p_out);
-
       p_out = p_next;
     }
 
   node_pointer p_cur = base_type::m_p_root;
-
   m_p_max = base_type::m_p_root = 0;
-
   while (p_cur != 0)
     {
       node_pointer p_next = p_cur->m_p_next_sibling;
-
       make_root_and_link(p_cur);
-
       p_cur = p_next;
     }
 
-  _GLIBCXX_DEBUG_ONLY(assert_valid();)
-
-    return ersd;
+  PB_DS_ASSERT_VALID((*this))
+  return ersd;
 }
 
 PB_DS_CLASS_T_DEC
@@ -285,12 +242,14 @@ inline typename PB_DS_CLASS_C_DEC::size_type
 PB_DS_CLASS_C_DEC::
 rank_bound()
 {
-  const std::size_t* const p_upper =
-    std::upper_bound(            g_a_rank_bounds, g_a_rank_bounds + num_distinct_rank_bounds, base_type::m_size);
+  using namespace std;
+  const size_t* const p_upper =
+    std::upper_bound(g_a_rank_bounds,
+		     g_a_rank_bounds + num_distinct_rank_bounds,
+		     base_type::m_size);
 
   if (p_upper == g_a_rank_bounds + num_distinct_rank_bounds)
     return max_rank;
 
   return (p_upper - g_a_rank_bounds);
 }
-
