@@ -1,6 +1,6 @@
 // Verbose terminate_handler -*- C++ -*-
 
-// Copyright (C) 2001-2013 Free Software Foundation, Inc.
+// Copyright (C) 2001-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -78,7 +78,7 @@ _GLIBCXX_BEGIN_NAMESPACE_VERSION
 	// If the exception is derived from std::exception, we can
 	// give more information.
 	__try { __throw_exception_again; }
-#ifdef __EXCEPTIONS
+#if __cpp_exceptions
 	__catch(const exception& exc)
 	  {
 	    char const *w = exc.what();
