@@ -1,6 +1,6 @@
 // 2004-03-02  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2004-2014 Free Software Foundation, Inc.
+// Copyright (C) 2004-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -20,18 +20,9 @@
 // 27.4.3 fpos
 
 #include <iterator>
+#include <sstream>
 #include <testsuite_hooks.h>
 
-class Fred
-{
-public:
-  Fred(bool)
-  { }
-};
-
-void barney(Fred)
-{ }
-
 // libstdc++/14320
 void test01()
 {
@@ -39,9 +30,11 @@ void test01()
   bool test __attribute__((unused)) = true;
 
   typedef istreambuf_iterator<char>::difference_type Distance;
-  
-  Distance d = 0;
-  barney(d);
+	
+  Distance d;
+  istringstream in("5");
+  in >> d;
+  VERIFY( d == 5 );
 }
 
 int main()
