@@ -26,8 +26,8 @@ int main()
 {
   using namespace __gnu_test;
 
-  // cplus-dem FAIL
-  verify_demangle("_Z1fPKM1AFivE", "f(int (A::* const*)())");
+  // cplus-dem FAIL  
+  verify_demangle("_Z1jM1AFivEPS1_", "j(int (A::*)(), int (A::**)())");
 
   return 0;
 }
