@@ -1,6 +1,6 @@
 // 2003-05-01 Petur Runolfsson <peturr02@ru.is>
 
-// Copyright (C) 2003-2014 Free Software Foundation, Inc.
+// Copyright (C) 2003-2015 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -23,25 +23,25 @@
 #include <testsuite_hooks.h>
 
 // libstdc++/12048
-void test02()
+void test05()
 {
   bool test __attribute__((unused)) = true;
   const char* name = "cin_unget-1.txt";
 
   std::FILE* file = std::fopen(name, "r");
   __gnu_cxx::stdio_sync_filebuf<char> sbuf(file);
-  int c1 = sbuf.sbumpc();
-  VERIFY( c1 != EOF );
+  char buf[2];
+  VERIFY( sbuf.sgetn(buf, 2) == 2 );
   int c2 = sbuf.sungetc();
   VERIFY( c2 != EOF );
-  int c3 = sbuf.sbumpc();
-  VERIFY( c3 == c1 );
+  int c3 = std::fgetc(file);
+  VERIFY( c3 == std::char_traits<char>::to_int_type(buf[1]) );
 
   std::fclose(file);
 }
 
 int main ()
 {
-  test02();
+  test05();
   return 0;
 }
